#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import logging
import subprocess
import json
import shutil
import fosslight_util.constant as constant
import fosslight_dependency.constant as const
from fosslight_dependency._package_manager import PackageManager
from fosslight_dependency._package_manager import check_and_run_license_scanner

logger = logging.getLogger(constant.LOGGER_NAME)


class Pypi(PackageManager):
    package_manager_name = const.PYPI

    dn_url = 'https://pypi.org/project/'
    venv_tmp_dir = 'venv_osc_dep_tmp'
    tmp_file_name = "tmp_pip_license_output.json"
    tmp_pip_license_info_file_name = "tmp_pip_license_info_output.json"
    pip_activate_cmd = ''
    pip_deactivate_cmd = ''

    def __init__(self, input_dir, output_dir, pip_activate_cmd, pip_deactivate_cmd):
        super().__init__(self.package_manager_name, self.dn_url, input_dir, output_dir)

        self.pip_activate_cmd = pip_activate_cmd
        self.pip_deactivate_cmd = pip_deactivate_cmd

    def __del__(self):
        if os.path.isfile(self.tmp_file_name):
            os.remove(self.tmp_file_name)

        if os.path.isfile(self.tmp_pip_license_info_file_name):
            os.remove(self.tmp_pip_license_info_file_name)

        if os.path.isdir(self.venv_tmp_dir):
            shutil.rmtree(self.venv_tmp_dir)

    def set_pip_activate_cmd(self, pip_activate_cmd):
        self.pip_activate_cmd = pip_activate_cmd

    def set_pip_deactivate_cmd(self, pip_deactivate_cmd):
        self.pip_deactivate_cmd = pip_deactivate_cmd

    def run_plugin(self):
        ret = True

        if not self.pip_activate_cmd and not self.pip_deactivate_cmd:
            ret = self.create_virtualenv()

        if ret:
            ret = self.start_pip_licenses()

        return ret

    def create_virtualenv(self):
        ret = True

        manifest_file = const.SUPPORT_PACKAE.get(self.package_manager_name)
        if not os.path.isfile(manifest_file):
            logger.error('Cannot create virtualenv becuase it cannot find the ' + manifest_file)
            logger.error("Please run with '-a' and '-d' option.")
            return False

        venv_path = os.path.join(self.input_dir, self.venv_tmp_dir)

        if self.platform == const.WINDOWS:
            create_venv_cmd = "python -m venv " + self.venv_tmp_dir
            activate_cmd = os.path.join(self.venv_tmp_dir, "Scripts", "activate.bat")
            cmd_separator = "&"
        else:
            create_venv_cmd = "virtualenv -p python3 " + self.venv_tmp_dir
            activate_cmd = ". " + os.path.join(venv_path, "bin", "activate")
            cmd_separator = ";"

        install_cmd = "pip install -r requirements.txt"
        deactivate_cmd = "deactivate"

        self.set_pip_activate_cmd(activate_cmd)
        self.set_pip_deactivate_cmd(deactivate_cmd)

        cmd_list = [create_venv_cmd, activate_cmd, install_cmd, deactivate_cmd]
        cmd = cmd_separator.join(cmd_list)

        try:
            cmd_ret = subprocess.call(cmd, shell=True)
            if cmd_ret != 0:
                ret = False
                err_msg = 'return code(' + str(cmd_ret) + ')'
        except Exception as e:
            ret = False
            err_msg = e
        finally:
            if ret:
                logger.info('It created the temporary virtualenv(' + venv_path + ').')
            else:
                logger.error('Failed to create virtualenv:' + str(err_msg))

        return ret

    def start_pip_licenses(self):
        ret = True
        pip_licenses = 'pip-licenses'
        ptable = 'PTable'
        pip_licenses_default_options = ' --from=mixed --with-url --format=json --with-license-file'
        pip_licenses_system_option = ' --with-system -p '
        pip_license_dependency = [pip_licenses, ptable]
        tmp_pip_list = "tmp_list.txt"

        if self.pip_activate_cmd.startswith("source "):
            tmp_activate = self.pip_activate_cmd[7:]
            pip_activate_cmd = ". " + tmp_activate
        elif self.pip_activate_cmd.startswith("conda "):
            if self.platform == const.LINUX:
                tmp_activate = "eval \"$(conda shell.bash hook)\";"
                pip_activate_cmd = tmp_activate + self.pip_activate_cmd
        else:
            pip_activate_cmd = self.pip_activate_cmd

        if self.platform == const.WINDOWS:
            command_separator = "&"
        else:
            command_separator = ";"

        activate_command = pip_activate_cmd
        pip_list_command = "pip freeze > " + tmp_pip_list
        deactivate_command = self.pip_deactivate_cmd

        command_list = [activate_command, pip_list_command, deactivate_command]
        command = command_separator.join(command_list)

        try:
            cmd_ret = subprocess.call(command, shell=True)
            if cmd_ret != 0:
                ret = False
                err_msg = 'cmd ret code(' + str(cmd_ret) + ')'
        except Exception as e:
            ret = False
            err_msg = str(e)
        finally:
            if not ret:
                logger.error("Failed to freeze dependencies (" + command + "):" + err_msg)
                return False

        exists_pip_licenses = False
        exists_ptable = False

        if os.path.isfile(tmp_pip_list):
            try:
                with open(tmp_pip_list, 'r', encoding='utf-8') as pip_list_file:
                    for pip_list in pip_list_file.readlines():
                        pip_list_name = pip_list.split('==')[0]
                        if pip_list_name == pip_licenses:
                            exists_pip_licenses = True
                        elif pip_list_name == ptable:
                            exists_ptable = True
                os.remove(tmp_pip_list)
            except Exception as e:
                logger.error("Failed to read freezed package list file:" + str(e))
                return False

        command_list = []
        command_list.append(activate_command)
        if not exists_pip_licenses:
            install_pip_command = "pip install " + pip_licenses
            command_list.append(install_pip_command)

        pip_licenses_command = pip_licenses + pip_licenses_default_options + " > " + self.tmp_file_name
        command_list.append(pip_licenses_command)

        if exists_ptable:
            pip_licenses_info_command = pip_licenses + pip_licenses_default_options + pip_licenses_system_option
            if exists_pip_licenses:
                pip_licenses_info_command += " ".join(pip_license_dependency)
            else:
                pip_licenses_info_command += ptable
            pip_licenses_info_command += " > " + self.tmp_pip_license_info_file_name
            command_list.append(pip_licenses_info_command)

        if not exists_pip_licenses:
            uninstall_pip_command = "pip uninstall -y "
            if exists_ptable:
                uninstall_pip_command += pip_licenses
            else:
                uninstall_pip_command += " ".join(pip_license_dependency)
            command_list.append(uninstall_pip_command)

        command_list.append(deactivate_command)
        command = command_separator.join(command_list)

        try:
            cmd_ret = subprocess.call(command, shell=True)
            if cmd_ret == 0:
                self.append_input_package_list_file(self.tmp_file_name)
                if exists_ptable:
                    self.append_input_package_list_file(self.tmp_pip_license_info_file_name)
            else:
                logger.error("Failed to run pip-licenses:" + command)
                ret = False
        except Exception as e:
            ret = False
            logger.error("Failed to install/uninstall pip-licenses:" + str(e))

        return ret

    def parse_oss_information(self, f_name):
        sheet_list = []
        try:
            with open(f_name, 'r', encoding='utf-8') as json_file:
                json_data = json.load(json_file)

            for d in json_data:
                oss_init_name = d['Name']
                oss_name = self.package_manager_name + ":" + oss_init_name
                license_name = check_UNKNOWN(d['License'])
                homepage = check_UNKNOWN(d['URL'])
                oss_version = d['Version']
                dn_loc = self.dn_url + oss_init_name + "/" + oss_version

                if license_name is not None:
                    license_name = license_name.replace(';', ',')

                license_file_dir = d['LicenseFile']
                license_name_with_license_scanner = check_and_run_license_scanner(self.platform,
                                                                                  self.license_scanner_bin,
                                                                                  license_file_dir)

                if license_name_with_license_scanner != "":
                    license_name = license_name_with_license_scanner

                sheet_list.append([const.SUPPORT_PACKAE.get(self.package_manager_name),
                                   oss_name, oss_version,
                                   license_name, dn_loc, homepage, '', '', ''])

        except Exception as ex:
            logger.error("Failed to parse oss information: " + str(ex))

        return sheet_list


def check_UNKNOWN(text):
    if text == ['UNKNOWN'] or text == 'UNKNOWN':
        text = ""
    return text



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import logging
import json
import re
import fosslight_util.constant as constant
import fosslight_dependency.constant as const
from fosslight_dependency._package_manager import PackageManager
from fosslight_dependency._package_manager import check_and_run_license_scanner

logger = logging.getLogger(constant.LOGGER_NAME)


class Pub(PackageManager):
    package_manager_name = const.PUB

    dn_url = 'https://pub.dev/packages/'
    input_file_name = os.path.join('lib', 'oss_licenses.dart')

    def __init__(self, input_dir, output_dir):
        super().__init__(self.package_manager_name, self.dn_url, input_dir, output_dir)
        self.append_input_package_list_file(self.input_file_name)

    def parse_oss_information(self, f_name):
        tmp_license_txt_file_name = 'tmp_license.txt'
        json_data = ''

        with open(f_name, 'r', encoding='utf8') as pub_file:
            json_txt = preprocess_pub_result(pub_file)
            if json_txt:
                json_data = json.loads(json_txt)

        try:
            sheet_list = []

            for key in json_data:
                oss_origin_name = json_data[key]['name']
                oss_name = self.package_manager_name + ":" + oss_origin_name
                oss_version = json_data[key]['version']
                homepage = json_data[key]['homepage']
                dn_loc = self.dn_url + oss_origin_name + "/versions/" + oss_version
                license_txt = json_data[key]['license']

                tmp_license_txt = open(tmp_license_txt_file_name, 'w', encoding='utf-8')
                tmp_license_txt.write(license_txt)
                tmp_license_txt.close()

                license_name_with_license_scanner = check_and_run_license_scanner(self.platform,
                                                                                  self.license_scanner_bin,
                                                                                  tmp_license_txt_file_name)

                if license_name_with_license_scanner != "":
                    license_name = license_name_with_license_scanner
                else:
                    license_name = ''

                sheet_list.append([const.SUPPORT_PACKAE.get(self.package_manager_name),
                                  oss_name, oss_version, license_name, dn_loc, homepage, '', '', ''])

        except Exception as e:
            logger.error("Fail to parse pub oss information: " + str(e))

        if os.path.isfile(tmp_license_txt_file_name):
            os.remove(tmp_license_txt_file_name)

        return sheet_list


def preprocess_pub_result(input_file):
    matched_json = re.findall(r'final ossLicenses = <String, dynamic>({[\s\S]*});', input_file.read())
    if len(matched_json) > 0:
        return matched_json[0]
    else:
        logger.error("Fail to parse the result json from pub input file.")
        return ''



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import logging
import subprocess
import json
import shutil
import fosslight_util.constant as constant
import fosslight_dependency.constant as const
from fosslight_dependency._package_manager import PackageManager

logger = logging.getLogger(constant.LOGGER_NAME)


class Npm(PackageManager):
    package_manager_name = const.NPM

    dn_url = 'https://www.npmjs.com/package/'
    input_file_name = 'tmp_npm_license_output.json'

    def __init__(self, input_dir, output_dir):
        super().__init__(self.package_manager_name, self.dn_url, input_dir, output_dir)

    def __del__(self):
        if os.path.isfile(self.input_file_name):
            os.remove(self.input_file_name)

    def run_plugin(self):
        ret = self.start_license_checker()
        return ret

    def start_license_checker(self):
        ret = True
        tmp_custom_json = 'custom.json'
        flag_tmp_node_modules = False
        license_checker_cmd = 'license-checker --production --json --out ' + self.input_file_name
        custom_path_option = ' --customPath '
        node_modules = 'node_modules'
        npm_install_cmd = 'npm install --prod'

        if os.path.isdir(node_modules) != 1:
            logger.info("node_modules directory is not existed. So it executes 'npm install'.")
            flag_tmp_node_modules = True
            cmd_ret = subprocess.call(npm_install_cmd, shell=True)
            if cmd_ret != 0:
                logger.error(npm_install_cmd + " returns an error")
                return False

        # customized json file for obtaining specific items with license-checker
        self.make_custom_json(tmp_custom_json)

        cmd = license_checker_cmd + custom_path_option + tmp_custom_json
        cmd_ret = subprocess.call(cmd, shell=True)
        if cmd_ret != 0:
            logger.error("It returns the error: " + cmd)
            logger.error("Please check if the license-checker is installed.(sudo npm install -g license-checker)")
            return False
        else:
            self.append_input_package_list_file(self.input_file_name)

        os.remove(tmp_custom_json)
        if flag_tmp_node_modules:
            shutil.rmtree(node_modules)

        return ret

    def make_custom_json(self, tmp_custom_json):
        with open(tmp_custom_json, 'w', encoding='utf8') as custom:
            custom.write(
                "{\n\t\"name\": \"\",\n\t\"version\": \"\",\n\t\"licenses\": \"\",\n\t\"repository\": \
                \"\",\n\t\"url\": \"\",\n\t\"copyright\": \"\",\n\t\"licenseText\": \"\"\n}\n".encode().decode("utf-8"))

    def parse_oss_information(self, f_name):
        with open(f_name, 'r', encoding='utf8') as json_file:
            json_data = json.load(json_file)

        sheet_list = []

        keys = [key for key in json_data]

        for i in range(0, len(keys)):
            d = json_data.get(keys[i - 1])
            oss_init_name = d['name']
            oss_name = self.package_manager_name + ":" + oss_init_name

            if d['licenses']:
                license_name = d['licenses']
            else:
                license_name = ''

            oss_version = d['version']

            if d['repository']:
                dn_loc = d['repository']
            else:
                dn_loc = self.dn_url + oss_init_name + '/v/' + oss_version

            homepage = self.dn_url + oss_init_name

            multi_license = check_multi_license(license_name)

            if multi_license:
                for l_idx in range(0, len(license_name)):
                    license_name = license_name[l_idx].replace(",", "")

                    sheet_list.append([const.SUPPORT_PACKAE.get(self.package_manager_name),
                                      oss_name, oss_version, license_name, dn_loc, homepage, '', '', ''])
            else:
                license_name = license_name.replace(",", "")

                sheet_list.append([const.SUPPORT_PACKAE.get(self.package_manager_name),
                                  oss_name, oss_version, license_name, dn_loc, homepage, '', '', ''])

        return sheet_list


def check_multi_license(license_name):
    if isinstance(license_name, list):
        multi_license = True
    else:
        multi_license = False

    return multi_license



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import logging
import json
import yaml
import re
import fosslight_util.constant as constant
import fosslight_dependency.constant as const
from fosslight_dependency._package_manager import PackageManager

logger = logging.getLogger(constant.LOGGER_NAME)

_spec_repos = 'SPEC REPOS'
_source_type = ['git', 'http', 'svn', 'hg']


class Cocoapods(PackageManager):
    package_manager_name = const.COCOAPODS

    dn_url = 'https://cocoapods.org/'
    input_file_name = const.SUPPORT_PACKAE.get(package_manager_name)

    def __init__(self, input_dir, output_dir):
        super().__init__(self.package_manager_name, self.dn_url, input_dir, output_dir)
        self.append_input_package_list_file(self.input_file_name)

    def parse_oss_information(self, f_name):
        with open(f_name, 'r', encoding='utf8') as input_fp:
            podfile_yaml = yaml.load(input_fp, Loader=yaml.FullLoader)

        pod_in_sepc_list = []
        pod_not_in_spec_list = []
        spec_repo_list = []

        for spec_item_key in podfile_yaml[_spec_repos]:
            for spec_item in podfile_yaml[_spec_repos][spec_item_key]:
                spec_repo_list.append(spec_item)

        for pods_list in podfile_yaml['PODS']:
            if not isinstance(pods_list, str):
                for pods_list_key, pods_list_item in pods_list.items():
                    pod_in_sepc_list, spec_repo_list, pod_not_in_spec_list = \
                        compile_pods_item(pods_list_key, spec_repo_list, pod_in_sepc_list, pod_not_in_spec_list)
            else:
                pod_in_sepc_list, spec_repo_list, pod_not_in_spec_list = \
                    compile_pods_item(pods_list, spec_repo_list, pod_in_sepc_list, pod_not_in_spec_list)

        if len(spec_repo_list) != 0:
            for spec_in_item in spec_repo_list:
                spec_oss_name_adding_core = spec_in_item + "/Core"
                for pod_not_item in pod_not_in_spec_list:
                    if spec_oss_name_adding_core == pod_not_item[0]:
                        pod_in_sepc_list.append([spec_in_item, pod_not_item[1]])

        sheet_list = []

        for pod_oss in pod_in_sepc_list:

            search_oss_name = ""
            for alphabet_oss in pod_oss[0]:
                if not alphabet_oss.isalnum():
                    search_oss_name += "\\\\" + alphabet_oss
                else:
                    search_oss_name += alphabet_oss

            command = 'pod spec which --regex ' + '^' + search_oss_name + '$'
            spec_which = os.popen(command).readline()
            if spec_which.startswith('[!]'):
                logger.error("This command(" + command + ") returns an error")
                return ''

            file_path = spec_which.rstrip().split(os.path.sep)
            if file_path[0] == '':
                file_path_without_version = os.path.join(os.sep, *file_path[:-2])
            else:
                file_path_without_version = os.path.join(*file_path[:-2])
            spec_file_path = os.path.join(file_path_without_version, pod_oss[1], file_path[-1])

            with open(spec_file_path, 'r', encoding='utf8') as json_file:
                json_data = json.load(json_file)

                oss_origin_name = json_data['name']
                oss_name = self.package_manager_name + ":" + oss_origin_name
                oss_version = json_data['version']
                homepage = self.dn_url + 'pods/' + oss_origin_name

                if not isinstance(json_data['license'], str):
                    if 'type' in json_data['license']:
                        license_name = json_data['license']['type']
                else:
                    license_name = json_data['license']

                license_name = license_name.replace(",", "")

                source_keys = [key for key in json_data['source']]
                for src_type_i in _source_type:
                    if src_type_i in source_keys:
                        dn_loc = json_data['source'][src_type_i]
                        if dn_loc.endswith('.git'):
                            dn_loc = dn_loc[:-4]

                sheet_list.append([const.SUPPORT_PACKAE.get(self.package_manager_name),
                                  oss_name, oss_version, license_name, dn_loc, homepage, '', '', ''])

        return sheet_list


def compile_pods_item(pods_item, spec_repo_list, pod_in_sepc_list, pod_not_in_spec_list):
    pods_item_re = re.findall(r'(\S*)\s{1}\((.*)\)', pods_item)

    oss_name = pods_item_re[0][0]
    oss_version = pods_item_re[0][1]

    oss_info = []
    oss_info.append(oss_name)
    oss_info.append(oss_version)

    if oss_name in spec_repo_list:
        pod_in_sepc_list.append(oss_info)
        spec_repo_list.remove(oss_name)
    else:
        pod_not_in_spec_list.append(oss_info)

    return pod_in_sepc_list, spec_repo_list, pod_not_in_spec_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
from fosslight_util.help import PrintHelpMsg

_HELP_MESSAGE_DEPENDENCY = """
    Usage: fosslight_dependency [option1] <arg1> [option2] <arg2>...

    FOSSLight Dependency Scanner is the tool that supports the analysis of dependencies for multiple package managers.
    It detects the manifest file of package managers automatically and analyzes the dependencies with using open source tools.
    Then, it generates the report file that contains OSS information of dependencies.

    Currently, it supports the following package managers:
        Gradle (Java)
        Maven (Java)
        NPM (Node.js)
        PIP (Python)
        Pub (Dart with flutter)
        Cocoapods (Swift/Obj-C)
        Swift (Swift)
        Carthage (Swift/Obj-C)

    Options:
        Optional
            -h\t\t\t\t    Print help message.
            -v\t\t\t\t    Print the version of the script.
            -m <package_manager>\t    Enter the package manager.
                                        \t(npm, maven, gradle, pip, pub, cocoapods, android, swift, carthage)
            -p <input_path>\t\t    Enter the path where the script will be run.
            -o <output_path>\t\t    Output path
            \t\t\t\t\t(If you want to generate the specific file name, add the output path with file name.)
            -f <format>\t\t\t    Output file format (excel, csv, opossum)

        Required only for pypi
            -a <activate_cmd>\t\t    Virtual environment activate command(ex, 'conda activate (venv name)')
            -d <deactivate_cmd>\t\t    Virtual environment deactivate command(ex, 'conda deactivate')

        Required only for swift, carthage
            -t <token>\t\t\t    Enter the github personal access token.

        Optional only for gradle, maven
            -c <dir_name>\t\t    Enter the customized build output directory name
                                    \t\t-Default name : 'build' for gradle, 'target' for maven

        Optional only for android
            -n <app_name>\t\t    Enter the application directory name where the plugin output file is located(default: app)
        """


def print_help_msg():
    helpMsg = PrintHelpMsg(_HELP_MESSAGE_DEPENDENCY)
    helpMsg.print_help_msg(True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2020 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import sys
import argparse
import pkg_resources
import warnings
from datetime import datetime
import logging
import fosslight_dependency.constant as const
from fosslight_util.set_log import init_log
import fosslight_util.constant as constant
from fosslight_dependency._help import print_help_msg
from fosslight_dependency._analyze_dependency import analyze_dependency
from fosslight_util.output_format import check_output_format, write_output_file

# Package Name
_PKG_NAME = "fosslight_dependency"
logger = logging.getLogger(constant.LOGGER_NAME)
warnings.filterwarnings("ignore", category=FutureWarning)
_sheet_name = "SRC_FL_Dependency"


def find_package_manager():
    ret = True
    manifest_file_name = list(set(list(const.SUPPORT_PACKAE.values())))

    found_manifest_file = []
    for f in manifest_file_name:
        if os.path.isfile(f):
            found_manifest_file.append(f)

    found_package_manager = []
    for f_idx in found_manifest_file:
        for key, value in const.SUPPORT_PACKAE.items():
            if value == f_idx:
                found_package_manager.append(key)

    if len(found_package_manager) >= 1:
        logger.info("Found the manifest file(" + ','.join(found_manifest_file) + ")automatically.")
        logger.warning("### Set Package Manager = " + ', '.join(found_package_manager))
    else:
        ret = False
        logger.error("Cannot find the manifest file.")
        logger.error("Please run with '-m' option to append the package manager name to be analzyed.")

    return ret, found_package_manager


def run_dependency_scanner(package_manager='', input_dir='', output_dir_file='', pip_activate_cmd='', pip_deactivate_cmd='',
                           output_custom_dir='', app_name=const.default_app_name, github_token='', format=''):
    global logger

    ret = True
    sheet_list = {}
    sheet_list[_sheet_name] = []
    _json_ext = ".json"
    _start_time = datetime.now().strftime('%y%m%d_%H%M%S')

    success, msg, output_path, output_file, output_extension = check_output_format(output_dir_file, format)
    if success:
        if output_path == "":
            output_path = os.getcwd()
        else:
            output_path = os.path.abspath(output_path)

        if output_file == "":
            if output_extension == _json_ext:
                output_file = "Opossum_input_" + _start_time
            else:
                output_file = "FOSSLight-Report_" + _start_time

    logger, _result_log = init_log(os.path.join(output_path, "fosslight_dependency_log_" + _start_time + ".txt"),
                                   True, logging.INFO, logging.DEBUG, _PKG_NAME)

    logger.info("Tool Info : " + _result_log["Tool Info"])

    if not success:
        logger.error(msg)
        return False, sheet_list

    autodetect = True
    if package_manager:
        autodetect = False
        support_packagemanager = list(const.SUPPORT_PACKAE.keys())

        if package_manager not in support_packagemanager:
            logger.error("You entered the unsupported package manager(" + package_manager + ").")
            logger.error("Please enter the supported package manager({0}) with '-m' option."
                         .format(", ".join(support_packagemanager)))
            return False, sheet_list

    if input_dir:
        if os.path.isdir(input_dir):
            os.chdir(input_dir)
            input_dir = os.getcwd()
        else:
            logger.error("You entered the wrong input path(" + input_dir + ") to run the script.")
            logger.error("Please enter the existed input path with '-p' option.")
            return False, sheet_list
    else:
        input_dir = os.getcwd()
        os.chdir(input_dir)

    found_package_manager = []
    if autodetect:
        try:
            ret, found_package_manager = find_package_manager()
        except Exception as e:
            logger.error(str(e))
            ret = False
        finally:
            if not ret:
                logger.error("Failed to detect package manager automatically.")
                return False, sheet_list
    else:
        found_package_manager.append(package_manager)

    for pm in found_package_manager:
        ret, package_sheet_list = analyze_dependency(pm, input_dir, output_path, pip_activate_cmd, pip_deactivate_cmd,
                                                     output_custom_dir, app_name, github_token)
        if ret:
            sheet_list[_sheet_name].extend(package_sheet_list)

    if sheet_list is not None:
        output_file_without_ext = os.path.join(output_path, output_file)
        success_to_write, writing_msg = write_output_file(output_file_without_ext, output_extension,
                                                          sheet_list)
        if success_to_write:
            logger.info("Writing Output file(" + output_file + output_extension + "):" + str(success_to_write) + " "
                                               + writing_msg)
        else:
            ret = False
            logger.error("Fail to generate result file. msg:()" + writing_msg)
    else:
        logger.error("Analyzing result is empty.")

    logger.warning("### FINISH ###")
    return ret, sheet_list


def main():
    package_manager = ''
    input_dir = ''
    output_dir = ''
    pip_activate_cmd = ''
    pip_deactivate_cmd = ''
    output_custom_dir = ''
    app_name = const.default_app_name
    github_token = ''
    format = ''

    parser = argparse.ArgumentParser(add_help=False)
    parser.add_argument('-h', '--help', action='store_true', required=False)
    parser.add_argument('-v', '--version', action='store_true', required=False)
    parser.add_argument('-m', '--manager', nargs=1, type=str, default='', required=False)
    parser.add_argument('-p', '--path', nargs=1, type=str, required=False)
    parser.add_argument('-o', '--output', nargs=1, type=str, required=False)
    parser.add_argument('-a', '--activate', nargs=1, type=str, default='', required=False)
    parser.add_argument('-d', '--deactivate', nargs=1, type=str, default='', required=False)
    parser.add_argument('-c', '--customized', nargs=1, type=str, required=False)
    parser.add_argument('-n', '--appname', nargs=1, type=str, required=False)
    parser.add_argument('-t', '--token', nargs=1, type=str, required=False)
    parser.add_argument('-f', '--format', nargs=1, type=str, required=False)

    args = parser.parse_args()

    if args.help:  # -h option
        print_help_msg()

    if args.version:  # -v option
        cur_version = pkg_resources.get_distribution(_PKG_NAME).version
        print('FOSSLight Dependency Scanner Version : ' + cur_version)
        sys.exit(0)

    if args.manager:  # -m option
        package_manager = ''.join(args.manager)
    if args.path:  # -p option
        input_dir = ''.join(args.path)
    if args.output:  # -o option
        output_dir = ''.join(args.output)
    if args.activate:  # -a option
        pip_activate_cmd = ''.join(args.activate)
    if args.deactivate:  # -d option
        pip_deactivate_cmd = ''.join(args.deactivate)
    if args.customized:  # -c option
        output_custom_dir = ''.join(args.customized)
    if args.appname:  # -n option
        app_name = ''.join(args.appname)
    if args.token:  # -t option
        github_token = ''.join(args.token)
    if args.format:  # -f option
        format = ''.join(args.format)

    run_dependency_scanner(package_manager, input_dir, output_dir, pip_activate_cmd, pip_deactivate_cmd,
                           output_custom_dir, app_name, github_token, format)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import logging
import fosslight_dependency.constant as const
from fosslight_dependency.package_manager.Pypi import Pypi
from fosslight_dependency.package_manager.Npm import Npm
from fosslight_dependency.package_manager.Maven import Maven
from fosslight_dependency.package_manager.Gradle import Gradle
from fosslight_dependency.package_manager.Pub import Pub
from fosslight_dependency.package_manager.Cocoapods import Cocoapods
from fosslight_dependency.package_manager.Android import Android
from fosslight_dependency.package_manager.Swift import Swift
from fosslight_dependency.package_manager.Carthage import Carthage
import fosslight_util.constant as constant

logger = logging.getLogger(constant.LOGGER_NAME)


def analyze_dependency(package_manager_name, input_dir, output_dir, pip_activate_cmd='', pip_deactivate_cmd='',
                       output_custom_dir='', app_name=const.default_app_name, github_token=''):
    ret = True
    package_sheet_list = []

    if package_manager_name == const.PYPI:
        package_manager = Pypi(input_dir, output_dir, pip_activate_cmd, pip_deactivate_cmd)
    elif package_manager_name == const.NPM:
        package_manager = Npm(input_dir, output_dir)
    elif package_manager_name == const.MAVEN:
        package_manager = Maven(input_dir, output_dir, output_custom_dir)
    elif package_manager_name == const.GRADLE:
        package_manager = Gradle(input_dir, output_dir, output_custom_dir)
    elif package_manager_name == const.PUB:
        package_manager = Pub(input_dir, output_dir)
    elif package_manager_name == const.COCOAPODS:
        package_manager = Cocoapods(input_dir, output_dir)
    elif package_manager_name == const.ANDROID:
        package_manager = Android(input_dir, output_dir, app_name)
    elif package_manager_name == const.SWIFT:
        package_manager = Swift(input_dir, output_dir, github_token)
    elif package_manager_name == const.CARTHAGE:
        package_manager = Carthage(input_dir, output_dir, github_token)
    else:
        logger.error("Not supported package manager name: " + package_manager_name)
        ret = False
        return ret, package_sheet_list

    ret = package_manager.run_plugin()
    if ret:
        for f_name in package_manager.input_package_list_file:
            logger.info("Parse oss information with file: " + str(f_name))

            if os.path.isfile(f_name):
                package_sheet_list = package_manager.parse_oss_information(f_name)
            else:
                logger.error('Failed to open input file:' + str(f_name))
                ret = False

    if ret:
        logger.warning("### Complete to analyze: " + package_manager_name)
    else:
        logger.error("### Fail to analyze: " + package_manager_name)

    return ret, package_sheet_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import logging
import json
import fosslight_util.constant as constant
import fosslight_dependency.constant as const
from fosslight_dependency._package_manager import PackageManager
from fosslight_dependency._package_manager import connect_github
from fosslight_dependency._package_manager import get_github_license

logger = logging.getLogger(constant.LOGGER_NAME)


class Swift(PackageManager):
    package_manager_name = const.SWIFT

    input_file_name = const.SUPPORT_PACKAE.get(package_manager_name)

    def __init__(self, input_dir, output_dir, github_token):
        super().__init__(self.package_manager_name, '', input_dir, output_dir)
        self.github_token = github_token

        self.check_input_file_path()
        self.append_input_package_list_file(self.input_file_name)

    def check_input_file_path(self):
        if not os.path.isfile(self.input_file_name):
            for file_in_swift in os.listdir("."):
                if file_in_swift.endswith(".xcodeproj"):
                    input_file_name_in_xcodeproj = os.path.join(file_in_swift,
                                                                "project.xcworkspace/xcshareddata/swiftpm",
                                                                self.input_file_name)
                    if input_file_name_in_xcodeproj != self.input_file_name:
                        if os.path.isfile(input_file_name_in_xcodeproj):
                            self.input_file_name = input_file_name_in_xcodeproj
                            logger.info("It uses the manifest file: " + self.input_file_name)

    def parse_oss_information(self, f_name):
        with open(f_name, 'r', encoding='utf8') as json_file:
            json_raw = json.load(json_file)
            json_data = json_raw["object"]["pins"]

        sheet_list = []

        g = connect_github(self.github_token)

        for key in json_data:
            oss_origin_name = key['package']
            oss_name = self.package_manager_name + ":" + oss_origin_name

            revision = key['state']['revision']
            version = key['state']['version']
            if version is None:
                oss_version = revision
            else:
                oss_version = version

            homepage = key['repositoryURL']
            dn_loc = homepage
            license_name = ''

            github_repo = "/".join(homepage.split('/')[-2:])
            license_name = get_github_license(g, github_repo, self.platform, self.license_scanner_bin)

            sheet_list.append([const.SUPPORT_PACKAE.get(self.package_manager_name),
                              oss_name, oss_version, license_name, dn_loc, homepage, '', '', ''])

        return sheet_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import logging
import subprocess
import shutil
from bs4 import BeautifulSoup as bs
from xml.etree.ElementTree import parse
import fosslight_util.constant as constant
import fosslight_dependency.constant as const
from fosslight_dependency._package_manager import PackageManager
from fosslight_dependency._package_manager import version_refine

logger = logging.getLogger(constant.LOGGER_NAME)


class Maven(PackageManager):
    package_manager_name = const.MAVEN

    dn_url = 'https://mvnrepository.com/artifact/'
    input_file_name = os.path.join('target', 'generated-resources', 'licenses.xml')
    is_run_plugin = False
    output_custom_dir = ''

    def __init__(self, input_dir, output_dir, output_custom_dir):
        super().__init__(self.package_manager_name, self.dn_url, input_dir, output_dir)

        if output_custom_dir:
            self.output_custom_dir = output_custom_dir
            self.input_file_name = os.path.join(output_custom_dir, os.sep.join(self.input_file_name.split(os.sep)[1:]))

        self.append_input_package_list_file(self.input_file_name)

    def __del__(self):
        if self.is_run_plugin:
            self.clean_run_maven_plugin_output()

    def run_plugin(self):
        ret = True

        if not os.path.isfile(self.input_file_name):
            self.is_run_plugin = True
            pom_backup = 'pom.xml_backup'

            ret = self.add_plugin_in_pom(pom_backup)
            if ret:
                self.run_maven_plugin()

            if os.path.isfile(pom_backup):
                shutil.move(pom_backup, const.SUPPORT_PACKAE.get(self.package_manager_name))

        return ret

    def add_plugin_in_pom(self, pom_backup):
        ret = False
        xml = 'xml'

        manifest_file = const.SUPPORT_PACKAE.get(self.package_manager_name)
        if os.path.isfile(manifest_file) != 1:
            logger.error(manifest_file + " is not existed in this directory.")
            return ret

        try:
            shutil.move(manifest_file, pom_backup)

            license_maven_plugin = '<plugin>\
                                            <groupId>org.codehaus.mojo</groupId>\
                                            <artifactId>license-maven-plugin</artifactId>\
                                            <version>2.0.0</version>\
                                            <executions>\
                                                <execution>\
                                                    <id>aggregate-download-licenses</id>\
                                                    <goals>\
                                                        <goal>aggregate-download-licenses</goal>\
                                                    </goals>\
                                                </execution>\
                                            </executions>\
                                            <configuration>\
                                                <excludedScopes>test</excludedScopes>\
                                            </configuration>\
                                        </plugin>'

            tmp_plugin = bs(license_maven_plugin, xml)

            license_maven_plugins = '<plugins>' + license_maven_plugin + '<plugins>'
            tmp_plugins = bs(license_maven_plugins, xml)

            with open(pom_backup, 'r', encoding='utf8') as f:
                f_xml = f.read()
                f_content = bs(f_xml, xml)

                build = f_content.find('build')
                if build is not None:
                    plugins = build.find('plugins')
                    if plugins is not None:
                        plugins.append(tmp_plugin.plugin)
                        ret = True
                    else:
                        build.append(tmp_plugins.plugins)
                        ret = True
        except Exception as e:
            ret = False
            logger.error('Failed to add plugin in pom' + str(e))

        if ret:
            with open(manifest_file, "w", encoding='utf8') as f_w:
                f_w.write(f_content.prettify(formatter="minimal").encode().decode('utf-8'))

        return ret

    def clean_run_maven_plugin_output(self):
        directory_name = os.path.dirname(self.input_file_name)
        licenses_path = os.path.join(directory_name, 'licenses')
        if os.path.isdir(directory_name):
            if os.path.isdir(licenses_path):
                shutil.rmtree(licenses_path)
                os.remove(self.input_file_name)

            if len(os.listdir(directory_name)) == 0:
                shutil.rmtree(directory_name)

        top_path = self.input_file_name.split(os.sep)[0]
        if len(os.listdir(top_path)) == 0:
            shutil.rmtree(top_path)

    def run_maven_plugin(self):
        logger.info('Run maven license scanning plugin with temporary pom.xml')
        if os.path.isfile('mvnw') or os.path.isfile('mvnw.cmd'):
            if self.platform == const.WINDOWS:
                cmd = "mvnw.cmd"
            else:
                cmd = "./mvnw"
        else:
            cmd = "mvn"
        cmd += " license:aggregate-download-licenses"

        ret = subprocess.call(cmd, shell=True)
        if ret != 0:
            logger.error("Failed to run maven plugin: " + cmd)

    def parse_oss_information(self, f_name):
        with open(f_name, 'r', encoding='utf8') as input_fp:
            tree = parse(input_fp)

        root = tree.getroot()
        dependencies = root.find("dependencies")

        sheet_list = []

        for d in dependencies.iter("dependency"):
            groupid = d.findtext("groupId")
            artifactid = d.findtext("artifactId")
            version = d.findtext("version")
            oss_version = version_refine(version)

            oss_name = groupid + ":" + artifactid
            dn_loc = self.dn_url + groupid + "/" + artifactid + "/" + version
            homepage = self.dn_url + groupid + "/" + artifactid

            licenses = d.find("licenses")
            if len(licenses):
                license_names = []
                for key_license in licenses.iter("license"):
                    if key_license.findtext("name") is not None:
                        license_names.append(key_license.findtext("name").replace(",", ""))
                license_name = ', '.join(license_names)
            else:
                # Case that doesn't include License tag value.
                license_name = ''

            sheet_list.append([const.SUPPORT_PACKAE.get(self.package_manager_name),
                              oss_name, oss_version, license_name, dn_loc, homepage, '', '', ''])

        return sheet_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

# System platform name
WINDOWS = 'Windows'
LINUX = 'Linux'
MACOS = 'Darwin'

# Package manager name
PYPI = 'pypi'
NPM = 'npm'
MAVEN = 'maven'
GRADLE = 'gradle'
PUB = 'pub'
COCOAPODS = 'cocoapods'
ANDROID = 'android'
SWIFT = 'swift'
CARTHAGE = 'carthage'

# Supported package name and manifest file
SUPPORT_PACKAE = {
    PYPI: 'requirements.txt',
    NPM: 'package.json',
    MAVEN: 'pom.xml',
    GRADLE: 'build.gradle',
    PUB: 'pubspec.yaml',
    COCOAPODS: 'Podfile.lock',
    ANDROID: 'build.gradle',
    SWIFT: 'Package.resolved',
    CARTHAGE: 'Cartfile.resolved'
}

# default android app name
default_app_name = 'app'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import logging
import fosslight_util.constant as constant
import fosslight_dependency.constant as const
from fosslight_dependency._package_manager import PackageManager

logger = logging.getLogger(constant.LOGGER_NAME)

_plugin_output_file = 'android_dependency_output.txt'


class Android(PackageManager):
    package_manager_name = const.ANDROID

    app_name = const.default_app_name
    input_file_name = ''

    def __init__(self, input_dir, output_dir, app_name):
        super().__init__(self.package_manager_name, '', input_dir, output_dir)
        if app_name:
            self.app_name = app_name
        self.input_file_name = self.check_input_path(self.app_name, _plugin_output_file)
        self.append_input_package_list_file(self.input_file_name)

    def check_input_path(self, app_name, _plugin_output_file):
        if os.path.isfile(_plugin_output_file):
            return _plugin_output_file
        else:
            return os.path.join(app_name, _plugin_output_file)

    def parse_oss_information(self, f_name):
        with open(f_name, 'r', encoding='utf8') as input_fp:
            sheet_list = []

            for i, line in enumerate(input_fp.readlines()):
                split_str = line.strip().split("\t")
                if i < 2:
                    continue

                if len(split_str) == 9:
                    idx, manifest_file, oss_name, oss_version, license_name, dn_loc, homepage, NA, NA = split_str
                elif len(split_str) == 7:
                    idx, manifest_file, oss_name, oss_version, license_name, dn_loc, homepage = split_str
                else:
                    continue
                sheet_list.append([manifest_file, oss_name, oss_version, license_name, dn_loc, homepage, '', '', ''])

        return sheet_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import logging
import re
import fosslight_util.constant as constant
import fosslight_dependency.constant as const
from fosslight_dependency._package_manager import PackageManager
from fosslight_dependency._package_manager import connect_github
from fosslight_dependency._package_manager import get_github_license

logger = logging.getLogger(constant.LOGGER_NAME)


class Carthage(PackageManager):
    package_manager_name = const.CARTHAGE

    input_file_name = const.SUPPORT_PACKAE.get(package_manager_name)
    dn_url = "https://github.com/"

    def __init__(self, input_dir, output_dir, github_token):
        super().__init__(self.package_manager_name, self.dn_url, input_dir, output_dir)
        self.github_token = github_token
        self.append_input_package_list_file(self.input_file_name)

    def parse_oss_information(self, f_name):
        with open(f_name, 'r', encoding='utf8') as input_fp:
            sheet_list = []

            g = connect_github(self.github_token)

            for i, line in enumerate(input_fp.readlines()):

                re_result = re.findall(r'github[\s]\"(\S*)\"[\s]\"(\S*)\"', line)
                try:
                    github_repo = re_result[0][0]
                    oss_origin_name = github_repo.split('/')[1]
                    oss_name = self.package_manager_name + ":" + oss_origin_name
                    oss_version = re_result[0][1]
                    homepage = self.dn_url + github_repo
                    dn_loc = homepage

                    license_name = ''
                    license_name = get_github_license(g, github_repo, self.platform, self.license_scanner_bin)

                except Exception:
                    logger.error("It cannot find the github oss information. So skip it.")
                    break

                sheet_list.append([const.SUPPORT_PACKAE.get(self.package_manager_name),
                                  oss_name, oss_version, license_name, dn_loc, homepage, '', '', ''])

        return sheet_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import logging
import json
import fosslight_util.constant as constant
import fosslight_dependency.constant as const
from fosslight_dependency._package_manager import PackageManager
from fosslight_dependency._package_manager import version_refine

logger = logging.getLogger(constant.LOGGER_NAME)


class Gradle(PackageManager):
    package_manager_name = const.GRADLE

    dn_url = 'https://mvnrepository.com/artifact/'
    input_file_name = os.path.join('build', 'reports', 'license', 'dependency-license.json')

    def __init__(self, input_dir, output_dir, output_custom_dir):
        super().__init__(self.package_manager_name, self.dn_url, input_dir, output_dir)

        if output_custom_dir:
            self.output_custom_dir = output_custom_dir
            self.input_file_name = os.path.join(output_custom_dir, os.sep.join(self.input_file_name.split(os.sep)[1:]))

        self.append_input_package_list_file(self.input_file_name)

    def parse_oss_information(self, f_name):
        with open(f_name, 'r', encoding='utf8') as json_file:
            json_data = json.load(json_file)

        sheet_list = []

        for d in json_data['dependencies']:

            used_filename = False
            group_id = ""
            artifact_id = ""

            name = d['name']
            filename = d['file']

            if name != filename:
                group_id, artifact_id, oss_ini_version = parse_oss_name_version_in_artifactid(name)
                oss_name = group_id + ":" + artifact_id
            else:
                oss_name, oss_ini_version = parse_oss_name_version_in_filename(filename)
                used_filename = True

            oss_version = version_refine(oss_ini_version)

            license_names = []
            try:
                for licenses in d['licenses']:
                    if licenses['name'] != '':
                        license_names.append(licenses['name'].replace(",", ""))
                license_name = ', '.join(license_names)
            except Exception:
                logger.info("Cannot find the license name")

            if used_filename or group_id == "":
                dn_loc = 'Unknown'
                homepage = ''

            else:
                dn_loc = self.dn_url + group_id + "/" + artifact_id + "/" + oss_ini_version
                homepage = self.dn_url + group_id + "/" + artifact_id

            sheet_list.append([const.SUPPORT_PACKAE.get(self.package_manager_name),
                              oss_name, oss_version, license_name, dn_loc, homepage, '', '', ''])

        return sheet_list


def parse_oss_name_version_in_filename(name):
    filename = name.rstrip('.jar')
    split_name = filename.rpartition('-')

    oss_name = split_name[0]
    oss_version = split_name[2]

    return oss_name, oss_version


def parse_oss_name_version_in_artifactid(name):
    artifact_comp = name.split(':')

    group_id = artifact_comp[0]
    artifact_id = artifact_comp[1]
    oss_version = artifact_comp[2]

    return group_id, artifact_id, oss_version



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import sys
import logging
import platform
import re
import base64
import fosslight_util.constant as constant
import fosslight_dependency.constant as const

try:
    from github import Github
except Exception:
    pass

logger = logging.getLogger(constant.LOGGER_NAME)

# binary url to check license text
_license_scanner_linux = "third_party/nomos/nomossa"
_license_scanner_macos = "third_party/askalono/askalono_macos"
_license_scanner_windows = "third_party\\askalono\\askalono.exe"


class PackageManager:
    input_package_list_file = []

    def __init__(self, package_manager_name, dn_url, input_dir, output_dir):
        self.input_package_list_file = []
        self.package_manager_name = package_manager_name
        self.input_dir = input_dir
        self.output_dir = output_dir
        self.dn_url = dn_url

        self.platform = platform.system()
        self.license_scanner_bin = check_license_scanner(self.platform)

    def run_plugin(self):
        logger.info('This package manager(' + self.package_manager_name + ') skips the step to run plugin.')
        return True

    def append_input_package_list_file(self, input_package_file):
        self.input_package_list_file.append(input_package_file)


def version_refine(oss_version):
    version_cmp = oss_version.upper()

    if version_cmp.find(".RELEASE") != -1:
        oss_version = version_cmp.rstrip(".RELEASE")
    elif version_cmp.find(".FINAL") != -1:
        oss_version = version_cmp.rstrip(".FINAL")

    return oss_version


def connect_github(github_token):
    if github_token is not None:
        g = Github(github_token)
    else:
        g = Github()

    return g


def get_github_license(g, github_repo, platform, license_scanner_bin):
    license_name = ''
    tmp_license_txt_file_name = 'tmp_license.txt'

    try:
        repository = g.get_repo(github_repo)
    except Exception:
        logger.error("It cannot find the license name. Please use '-t' option with github token.")
        logger.error("{0}{1}".format("refer:https://docs.github.com/en/github/authenticating-to-github/",
                     "keeping-your-account-and-data-secure/creating-a-personal-access-token"))
        repository = ''

    if repository is not None:
        try:
            license_name = repository.get_license().license.spdx_id
        except Exception:
            logger.info("Cannot find the license name with github api.")

        if license_name == "" or license_name == "NOASSERTION":
            try:
                license_txt_data = base64.b64decode(repository.get_license().content).decode('utf-8')
                tmp_license_txt = open(tmp_license_txt_file_name, 'w', encoding='utf-8')
                tmp_license_txt.write(license_txt_data)
                tmp_license_txt.close()
                license_name = check_and_run_license_scanner(platform, license_scanner_bin, tmp_license_txt_file_name)
            except Exception:
                logger.info("Cannot find the license name with license scanner binary.")

            if os.path.isfile(tmp_license_txt_file_name):
                os.remove(tmp_license_txt_file_name)

    return license_name


def check_license_scanner(platform):
    license_scanner_bin = ''

    if platform == const.LINUX:
        license_scanner = _license_scanner_linux
    elif platform == const.MACOS:
        license_scanner = _license_scanner_macos
    elif platform == const.WINDOWS:
        license_scanner = _license_scanner_windows
    else:
        logger.debug("Not supported OS to analyze license text with binary.")

    if license_scanner:
        try:
            base_path = sys._MEIPASS
        except Exception:
            base_path = os.path.dirname(__file__)

        data_path = os.path.join(base_path, license_scanner)
        license_scanner_bin = data_path

    return license_scanner_bin


def check_and_run_license_scanner(platform, license_scanner_bin, file_dir):
    license_name = ''

    if not license_scanner_bin:
        logger.error('Not supported OS for license scanner binary.')

    try:
        tmp_output_file_name = "tmp_license_scanner_output.txt"

        if file_dir == "UNKNOWN":
            license_name = ""
        else:
            if platform == const.LINUX:
                run_license_scanner = license_scanner_bin + " " + file_dir + " > " + tmp_output_file_name
            elif platform == const.MACOS:
                run_license_scanner = license_scanner_bin + " identify " + file_dir + " > " + tmp_output_file_name
            elif platform == const.WINDOWS:
                run_license_scanner = license_scanner_bin + " identify " + file_dir + " > " + tmp_output_file_name
            else:
                run_license_scanner = ''

            if run_license_scanner is None:
                license_name = ""
                return license_name
            else:
                ret = os.system(run_license_scanner)
                if ret != 0:
                    logger.info("Cannot find the license name with scanner bin.")
                    return ""

            fp = open(tmp_output_file_name, "r", encoding='utf8')
            license_output = fp.read()
            fp.close()
            os.remove(tmp_output_file_name)

            if platform == const.LINUX:
                license_output_re = re.findall(r'.*contains license\(s\)\s(.*)', license_output)
            else:
                license_output_re = re.findall(r"License:\s{1}(\S*)\s{1}", license_output)

            if len(license_output_re) == 1:
                license_name = license_output_re[0]
                if license_name == "No_license_found":
                    license_name = ""
            else:
                license_name = ""

    except Exception as ex:
        logger.error("Failed to run license scan binary." + str(ex))
        license_name = ""

    return license_name



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# SPDX-License-Identifier: Apache-2.0
# Copyright (c) 2020 LG Electronics

from codecs import open
from setuptools import setup, find_packages

with open('README.md', 'r', 'utf-8') as f:
    reamdme = f.read()

with open('requirements.txt', 'r', 'utf-8') as f:
    required = f.read().splitlines()


if __name__ == "__main__":
    setup(
        name='fosslight_dependency',
        version='3.7.4',
        package_dir={"": "src"},
        packages=find_packages(where='src'),
        description='FOSSLight Dependency Scanner',
        long_description=reamdme,
        long_description_content_type='text/markdown',
        license='Apache-2.0',
        author='LG Electronics',
        url='https://github.com/fosslight/fosslight_dependency_scanner',
        download_url='https://github.com/fosslight/fosslight_dependency_scanner',
        classifiers=['Programming Language :: Python :: 3.6',
                     'License :: OSI Approved :: Apache Software License'],
        install_requires=required,
        package_data={'fosslight_dependency': ['third_party/nomos/nomossa',
                                               'third_party/askalono/askalono.exe',
                                               'third_party/askalono/askalono_macos']},
        include_package_data=True,
        entry_points={
            "console_scripts": [
                "fosslight_dependency = fosslight_dependency.run_dependency_scanner:main"
            ]
        }
    )
